// LED test fixture for PIN-10AP photodiode
// Ed Nisley KE4ZNU May 2013
// More details on my blog:
// http://softsolder.com/2013/05/23/led-photodiode-test-fixture/

Layout = "Show";		// Adapter AdapterSupport Cap Shield Build Show

Gap = 8;		// between parts in Show

//- Extrusion parameters must match reality!
//  Print with +1 shells and 3 solid layers

ThreadThick = 0.25;
ThreadWidth = 0.40;

HoleWindage = 0.2;

function IntegerMultiple(Size,Unit) = Unit * ceil(Size / Unit);

Protrusion = 0.1;			// make holes end cleanly

Spacing = 5;				// between parts on build platform

inch = 25.4;

Tap2_56 = 0.070 * inch;
Clear2_56 = 0.082 * inch;
Head2_56 = 0.156 * inch;
Head2_56Thick = 0.055 * inch;

//----------------------
// Dimensions

PhotoDiodeOD = 31.3;
PhotoDiodeStemOD = 16.0;
PhotoDiodeStemLength = 8.0;
PhotoDiodeWindowDia = 17.7;
PhotoDiodeHeight = 14.0;

FixtureOD = PhotoDiodeOD + 2*7.0;

LEDDia = 5.0;				// LED body
LEDFlangeOD = 6.0;			// flange at base of LED
LEDFlangeThick = IntegerMultiple(1.5,ThreadThick);
LEDLength = 10.0;			// overall length
LEDRecess = 4.0;			// tube to fit LED body
LEDSides = 8;

FixtureLength = PhotoDiodeHeight + LEDLength + IntegerMultiple(1.0,ThreadThick);

CapLength = 15.0;			// LED cover

FoamOD = FixtureOD/2;
FoamThick = IntegerMultiple(2.0,ThreadThick);

TrenchDepth = 2*FoamThick;
TrenchWidth = LEDDia;

ShieldThick = 5.0;
ShieldScrewCircle = PhotoDiodeOD + (FixtureOD - PhotoDiodeOD)/2;

PinOD = 3.0;				// alignment pin (filament)
PinLength = 10.0;			//   ... total length
PinCircle = FixtureOD/2;

GrubScrewOD = Tap2_56;

$fn=4*6;					// default cylinder sides

//----------------------
// Useful routines


module PolyCyl(Dia,Height,ForceSides=0) {			// based on nophead's polyholes

  Sides = (ForceSides != 0) ? ForceSides : (ceil(Dia) + 2);

  FixDia = Dia / cos(180/Sides);

  cylinder(r=(FixDia + HoleWindage)/2,
           h=Height,
	   $fn=Sides);
}

module ShowPegGrid(Space = 10.0,Size = 1.0) {

  RangeX = floor(95 / Space);
  RangeY = floor(125 / Space);

	for (x=[-RangeX:RangeX])
	  for (y=[-RangeY:RangeY])
		translate([x*Space,y*Space,Size/2])
		  %cube(Size,center=true);

}

//-----------------------
// Parts

module Adapter() {

	difference() {
		cylinder(r=FixtureOD/2,h=FixtureLength);
		translate([0,0,-Protrusion]) {
			PolyCyl(LEDDia,2*FixtureLength,LEDSides);
			PolyCyl(PhotoDiodeWindowDia,(FixtureLength - LEDRecess + Protrusion));
			PolyCyl(PhotoDiodeOD,(PhotoDiodeHeight + Protrusion));
		}
		translate([0,0,(FixtureLength - LEDFlangeThick)])
			PolyCyl(LEDFlangeOD,2*LEDFlangeThick,LEDSides);
		translate([FixtureOD/2,0,(FixtureLength + FoamThick/2 - LEDFlangeThick)]) {
			cube([FixtureOD,TrenchWidth,FoamThick],center=true);
		}
		for (angle = [90:90:270])
			rotate(angle)
				translate([0.75*PinCircle,0,(FixtureLength - PinLength/2)])
					PolyCyl(PinOD,PinLength,6);
		for (angle = [0:120:240])
			rotate(angle)
				translate([ShieldScrewCircle/2,0,-Protrusion])
					rotate(45)
						PolyCyl(Tap2_56,(ShieldThick - 6*ThreadThick + Protrusion));
		if (0)
			translate([0,0,FixtureLength/4])
				rotate([0,90,0])
					PolyCyl(GrubScrewOD,FixtureOD);
	}

}

module AdapterSupport() {

spiderthick = IntegerMultiple(LEDFlangeThick - ThreadThick,ThreadThick);

	color("Yellow")
		union() {
			for (leg = [0:LEDSides/2 - 1])
				rotate(leg*360/LEDSides)
					translate([0,0,spiderthick/2])
					cube([(LEDFlangeOD - 0.5*ThreadWidth),
							2.5*ThreadWidth,
							spiderthick],
							center=true);
			cylinder(r=LEDDia/2,h=spiderthick,$fn=LEDSides);
			for (bar = [-1:1])
				translate([LEDDia/3,(bar*3*ThreadWidth - ThreadWidth),0])
					cube([FixtureOD/2,2*ThreadWidth,IntegerMultiple(LEDFlangeThick - ThreadThick)]);
		}
}

module Cap() {

	difference() {
		cylinder(r=FixtureOD/2,h=CapLength);
		translate([(FixtureOD/2 - LEDDia/2),0,CapLength]) {
			cube([FixtureOD,TrenchWidth,2*TrenchDepth],center=true);
		}
		translate([0,0,(CapLength - FoamThick)])
			PolyCyl(FoamOD,(FoamThick + Protrusion));
		for (angle = [90:90:270])
			rotate(angle)
				translate([0.75*PinCircle,0,(CapLength - PinLength/2)])
					PolyCyl(PinOD,PinLength,6);
	}

}


module Shield() {

	difference() {
		cylinder(r=FixtureOD/2,h=ShieldThick);
		translate([0,0,-Protrusion])
			PolyCyl(PhotoDiodeStemOD,(ShieldThick + 2*Protrusion));
		for (angle = [0:120:240])
			rotate(angle) {
				translate([ShieldScrewCircle/2,0,-Protrusion])
					rotate(180/5)
						PolyCyl(Clear2_56,(ShieldThick + 2*Protrusion));
				if (0)
				translate([ShieldScrewCircle/2,0,(ShieldThick - 1.5*Head2_56Thick)])
					rotate(180/6)
						PolyCyl(Head2_56,4*Head2_56Thick);
			}
	}
}

//-------------------
// Build it...


ShowPegGrid();

if (Layout == "Adapter")
	Adapter();

if (Layout == "Cap")
	Cap();

if (Layout == "Shield")
	Shield();

if (Layout == "Show") {
	translate([0,0,(ShieldThick + Gap)]) {
		translate([0,0,FixtureLength + CapLength + Gap])
			rotate([180,0,0])
				Cap();
		Adapter();

		color("Orange")
		for (angle = [90:90:270])
			rotate(angle)
				translate([0.75*PinCircle,0,(FixtureLength + Gap - PinLength/2)])
					PolyCyl(PinOD,PinLength,6);
	}
	
	Shield();
}

	if (Layout == "AdapterSupport") {
	translate([0,0,FixtureLength])
		rotate([180,0,0])
			%Adapter();
	AdapterSupport();
}

if (Layout == "Build") {
	translate([(Spacing + FixtureOD),0,0]) {
		translate([0,0,FixtureLength])
			rotate([180,0,0])
				Adapter();
		AdapterSupport();
	}
	translate([0,0,0])
		Cap();

	translate([-(Spacing + FixtureOD),0,0])
		Shield();
}